#include <ostream>
using namespace std;

/** Represents playing cards for card games
  * @author Don Blaheta
  * @version 5 Nov 2013 */
class Card
{
  private:
    /** Rank i.e. number or face: ace=1, JQK = 11,12,13 */
    int rank;
    /** Suit represented by uppercase first letter of suit (CDHS) */
    char suit;

    /** Returns the char associated with the rank number.  For
      * internal use (which is why it's private). */
    char rankChar() {
      switch (rank) /* I haven't taught you switch statements
                       and won't require you to write them, but
                       if you're curious how this works, see p701
                       of the textbook (or for other examples, p93
                       of the Horstmann textbook we used in CMSC160). */
      {
        case 1: return 'A';
        case 10: return 'T';
        case 11: return 'J';
        case 12: return 'Q';
        case 13: return 'K';
        default: return rank + '0';
      }
    }

  public:
    /** Default constructor constructs dummy (invalid) value */
    Card (): rank(0), suit('N') {}
    /** Constructs a Card with given rank and given suit */
    Card (int r, char s);

    /** Returns the rank of this Card (A=1, JQK = 11,12,13 */
    int getRank() const;
    /** Returns the suit of this Card (CDHS) */
    char getSuit() const;

    /** Determines whether this Card has same rank/suit as given other Card */
    bool isEqualTo(const Card& other) const;


    bool operator==(const Card& other) const;
    friend ostream& operator<<(ostream& out, const Card& other);
};
