%{
#include "tokens.h"
#define YY_DECL  int UnciLexer::yylex()
#include "uncilexer.h"
%}

%option noyywrap
%option c++

ws		[\t\n ]+
onepunc		[{}()\[\];:]
oppunc		[\.~!+\-*&/%&^|?,=]
compop		(<=|>=|<|>|==|!=)

%%

\/\/[^\n]*\n		{ ws += '\n'; ++ lineno; }
      /* see http://ostermiller.org/findcomment.html */
\/\*([^*]|\*\**[^*\/])*\*\**\/	{ sync_from_comment(yytext); }

^#[^\n]*\n		{ add_string(yytext); ++ lineno; return HASHLINE; }
\'([^\'\n]|\\[^\n])\'		{ char *chp = &yytext[1];
				semval = deback_char(chp);
				return CHARLIT; }

\"([^\"\n\\]|\\[^\n])*\"	{ add_string(yytext); return STRLIT; }
 /* not currently handling raw strings */

 /* this line makes vim syntax highlighing happy: */
"\""			{ return OTHER; }

 /* things that look like idents could be keywords; look them up: */
[a-zA-Z_][a-zA-Z0-9_]*	{ return add_symbol(yytext); }

 /* some sort of number? let the C++ compiler sort it all out */
[0-9][a-zA-Z0-9_]*	{ add_string(yytext); return NUMLIT; }

\+-			{ return PLUSMINUS; }
{onepunc}		{ return YYText()[0]; }
{compop}		{ return COMPOP; }
{oppunc}		{ return OPPUNC; }
\n			{ ++ lineno; ws += '\n'; }
[\t ]+			{ ws += YYText(); }
.			{ return OTHER; }

%%

void UnciLexer::sync_from_comment(const string& s)
{
  int newlines = 0; 
  for (char ch : s)
  {
    if (ch == '\n')
    {
      ++newlines;
      ws += '\n';
    }
  }
  lineno += newlines;
  if (newlines == 0)
    ws += ' ';
}

void UnciLexer::initReserved()
{
  token_names[PLUSMINUS] = "'+-'";
  token_names[OPPUNC] = "operator/punctuation";
  token_names[COMPOP] = "comparison operator";

  token_names[STMTKEY] = "statement keyword";
  token_names[USING] = "'using'";
  token_names[TRUE] = "'true'";
  token_names[FALSE] = "'false'";
  token_names[CHECK] = "'check'";
  token_names[EXPECT] = "'expect'";
  token_names[THROW] = "'throw'";
  token_names[ABOUT] = "'about'";
  token_names[ARRAY] = "'array'";
  token_names[TRACE] = "'trace'";
  token_names[FIXTURE] = "'fixture'";
  token_names[TEST] = "'test'";
  token_names[TESTS] = "'tests'";
  token_names[SUITE] = "'suite'";
  token_names[SETUP] = "'setup'";
  token_names[TEARDOWN] = "'teardown'";

  token_names[IDENT] = "identifier";
  token_names[STRLIT] = "string literal";
  token_names[CHARLIT] = "char literal";
  token_names[NUMLIT] = "number literal";

  token_names[HASHLINE] = "hash line";
  token_names[OTHER] = "OTHER";

  reserved_words["for"] = STMTKEY;
  reserved_words["if"] = STMTKEY;
  reserved_words["else"] = STMTKEY;
  reserved_words["do"] = STMTKEY;
  reserved_words["while"] = STMTKEY;
  reserved_words["switch"] = STMTKEY;

  reserved_words["using"] = USING;
  reserved_words["true"] = TRUE;
  reserved_words["false"] = FALSE;
  reserved_words["check"] = CHECK;
  reserved_words["expect"] = EXPECT;
  reserved_words["throw"] = THROW;
  reserved_words["about"] = ABOUT;
  reserved_words["array"] = ARRAY;
  reserved_words["trace"] = TRACE;
  reserved_words["fixture"] = FIXTURE;
  reserved_words["test"] = TEST;
  reserved_words["tests"] = TESTS;
  reserved_words["suite"] = SUITE;
  reserved_words["setup"] = SETUP;
  reserved_words["teardown"] = TEARDOWN;
}


int yyFlexLexer::yylex()
{
  return -1;  // never called; should call derived version
}
