#ifndef _LEXER_H_
#define _LEXER_H_

#ifndef __FLEX_LEXER_H
//#define yyFlexLexer SampleAutoFlexLexer
#include <FlexLexer.h>
//#undef yyFlexLexer
#endif

//#define YY_DECL  int SampleLexer::yylex()

#include <cstdio>
#include <string>
#include <map>
#include <vector>
using namespace std;

#include "tokens.h"

class UnciLexer : public yyFlexLexer
{
  public:
    UnciLexer();
    UnciLexer(istream* is, string filename);
    virtual ~UnciLexer() {}

    inline string consumed_filename() const { return prev_filename; }
    inline string consumed_tokentext() const { return prev_tokentext; }
    inline string consumed_ws() const { return prev_ws; }
    /** returns line number at start of (prev) token */
    inline int consumed_lineno() const
    {
      return (prev_token == HASHLINE? prev_lineno - 1: prev_lineno);
    }
    inline long consumed_value() const { return prev_semval; }
    inline int consumed_token() const { return prev_token; }

    inline string peek_filename() const { return filename; }
    inline string peek_tokentext() const { return tokentext; }
    inline string peek_ws() const { return ws; }
    /** returns line number at start of (next) token */
    inline int peek_lineno() const
    {
      return (token == HASHLINE? lineno - 1: lineno);
    }
    inline long peek_value() const { return semval; }
    inline int peek_token() const { return token; }

    inline int peek() const { return token; }

    int consume();

  protected:
    int yylex ();  // provided by flex based on unci.ll

    /** uses given # line to update line number and filename */
    void hash_locate(const char *hash);

    /** searches given map for given value without modifying map.
      * Equivalent to map::operator[] const if there were such a thing. */
    static int find_or_bust(const map<string, int> &dict, const string &s);

    /** simultaneously add given string s to given tables dict and table */
    static int add_to_table(map<string, int> &dict, vector<string> &table,
                const string& s);

    /** adjust ws and lineno for multiline comments */
    void sync_from_comment(const string& s);

    int add_symbol(const string& s);
    void add_string(const string& s);

    void initReserved();

    /** converts given character literal (e.g. '\n') into corresponding
      * char value.  Input is char* pointing to first char after 
      * single quote.  Doesn't handle octal or unicode. */
    static int deback_char (char *charlit);

  public:
    string token_name(int i) const;

    inline string symbol(int i) const { return symbol_table_by_int[i]; }
    int symbol(const string& s) const;
    inline int num_symbols() const { return symbol_table_by_int.size(); }


    inline string get_string(int i) const { return string_table_by_int[i]; }
    int get_string_id(const string& s) const;
    inline int num_strings() const { return string_table_by_int.size(); }

    void print_token_table_line(FILE *) ;

  private:
    string filename;
    string ws;
    string tokentext;
    int lineno;
    long semval;
    int token;

    string prev_filename;
    string prev_ws;
    string prev_tokentext;
    int prev_lineno;
    long prev_semval;
    int prev_token;


    map<string, int> symbol_table;
    vector<string> symbol_table_by_int;
    map<string, int> string_table;
    vector<string> string_table_by_int;
    map<string, int> reserved_words;
    map<int, string> token_names;


};

#endif
