#ifndef _UNITLANGPARSER_H_
#define _UNITLANGPARSER_H_

#include <deque>
using namespace std;

#include "astnode.h"
#include "uncilexer.h"

class UnciParser 
{
  public:
    UnciParser (UnciLexer &l)
      : lexer(l) { }

    test_suite * parse ();

  protected:
    deque<ast_node *> parse_prologue();
    leaf_node * parse_leaf();
    simple_stmt_node * parse_using_stmt();

    fixture_node * parse_the_fixture_opt();
    deque<test_node *> parse_the_tests_opt();
    test_node * parse_test_opt();

    named_block_node * parse_named_block_opt(token_t head);
    block_node * parse_block();

    stmt_node * parse_stmt(bool opt = false);
    stmt_node * parse_stmt_opt() { return parse_stmt(true); }
    simple_stmt_node * parse_simple_stmt(bool opt = false);
    simple_stmt_node * parse_simple_stmt_opt()
                        { return parse_simple_stmt(true); }
    check_stmt_node * parse_check_stmt();
    control_stmt_node * parse_control_stmt();

    expectation_node * parse_expectation();
    about_expectation_node * parse_about_expectation();
    array_expectation_node * parse_array_expectation();
    compop_expectation_node * parse_compop_expectation();

    deque<leaf_node *> parse_junk(bool stmt_word_ok = true);
    deque<leaf_node *> parse_junk_tail(deque<leaf_node *>& so_far);
    deque<leaf_node *> parse_junk_opt();
    deque<leaf_node *> parse_junk_matched(leaf_node *);


    leaf_node * match(token_t tok);
    leaf_node * must_match(token_t tok, string errmsg);
    bool must_consume(token_t tok, string errmsg);

  private:
    void syn_fail(const string &expected = "");

    UnciLexer &lexer;
};

#endif
